require([
    "jira/issue",
    "jira/util/events",
    "jira/util/events/types",
], function (JiraIssue, Events, Types) {
    const STATUS_UNKNOWN = 0;
    const STATUS_NONE = 1;
    const STATUS_NEW = 2;
    const STATUS_CONNECTED = 3;
    const STATUS_ACTIVE = 4;
    const STATUS_CLOSED = 5;

    baseUrl = null;
    supportStatus = STATUS_UNKNOWN;
    processGet = false;
    processAction = false;
    issueId = null;
    inviteUrl = "";
    connectUrl = "";
    timerHandler = null;

    function initializeView() {
        updateStatus(STATUS_UNKNOWN);
        $("#getscreen-webpanel-start").on("click", onStartClick);
        $("#getscreen-webpanel-cancel").on("click", onCancelClick);
        requestGet();
    }

    Events.bind(Types.ISSUE_REFRESHED, function () {
        issueId = JiraIssue.getIssueId();
        initializeView();
    });

    AJS.toInit(function () {
        baseUrl = AJS.contextPath() + "/rest/getscreen/1.0";
        issueId = JiraIssue.getIssueId();
        initializeView();
    });

    function requestGet() {
        if (baseUrl && issueId) {
            if (!processGet) {
                processGet = true;
                request("get", function () {
                    processGet = false;
                });
            }
        }
    }

    function requestAction(path) {
        if (baseUrl && issueId) {
            if (!processAction) {
                processAction = true;
                request(path, function () {
                    processAction = false;
                });
            }
        }
    }

    function request(path, callback) {
        AJS.$.ajax(baseUrl + "/" + path, {
            type: "GET",
            dataType: "json",
            contentType: "application/json",
            data: { issueId: issueId },
        })
            .done(function (data) {
                $("#getscreen-webpanel-error").hide();
                $("#getscreen-webpanel-error").text("");
                inviteUrl = data.inviteUrl;
                connectUrl = data.connectUrl;
                updateStatus(data.status);
                if (!timerHandler) {
                    timerHandler = setInterval(function () {
                        requestGet();
                    }, 5000);
                }
            })
            .fail(function (data) {
                $("#getscreen-webpanel-error").show();
                $("#getscreen-webpanel-error").text(AJS.I18n.getText("me.getscreen.web.request.error"));
                if (timerHandler) {
                    clearInterval(timerHandler);
                    timerHandler = null;
                }
            })
            .always(function (data) {
                callback(null);
            });
    }

    function hideInvite(startText) {
        $("#getscreen-webpanel-text").show();
        // hide items
        $("#getscreen-webpanel-items").hide();
        $("#getscreen-webpanel-invite-url").text("");
        $("#getscreen-webpanel-invite-url").attr("href", "#");
        $("#getscreen-webpanel-status").text("");
        // reset start button
        $("#getscreen-webpanel-start").attr("href", "#");
        $("#getscreen-webpanel-start").text(startText);

        $("#getscreen-webpanel-cancel").hide();
    }

    function showInvite(startText, statusText) {
        $("#getscreen-webpanel-text").hide();
        // show items
        $("#getscreen-webpanel-items").show();
        $("#getscreen-webpanel-invite-url").text(inviteUrl);
        $("#getscreen-webpanel-invite-url").attr("href", inviteUrl);
        $("#getscreen-webpanel-status").text(statusText);
        // setup start button
        $("#getscreen-webpanel-start").attr("href", connectUrl);
        $("#getscreen-webpanel-start").text(startText);

        $("#getscreen-webpanel-cancel").show();
    }

    function showBalloon(checkStatus, text) {
        if (supportStatus != STATUS_UNKNOWN && supportStatus != checkStatus)
            JIRA.Messages.showSuccessMsg(text);
    }

    function updateStatus(status) {
        console.log("status = " + status);
        if (status == STATUS_UNKNOWN) {
            $("#getscreen-webpanel-spinner").show();
            $("#getscreen-webpanel-content").hide();
        } else {
            $("#getscreen-webpanel-spinner").hide();
            $("#getscreen-webpanel-content").show();
        }

        switch (status) {
            case STATUS_UNKNOWN:
                $("#getscreen-webpanel-cancel").hide();
                hideInvite(AJS.I18n.getText("me.getscreen.web.button.create"));
                break;

            case STATUS_NONE:
                $("#getscreen-webpanel-cancel").hide();
                hideInvite(AJS.I18n.getText("me.getscreen.web.button.create"));
                showBalloon(STATUS_NONE, AJS.I18n.getText("me.getscreen.web.baloon.closed"));
                break;

            case STATUS_CLOSED:
                $("#getscreen-webpanel-cancel").hide();
                hideInvite(AJS.I18n.getText("me.getscreen.web.button.create"));
                showBalloon(STATUS_CLOSED, AJS.I18n.getText("me.getscreen.web.baloon.closed"));
                break;

            case STATUS_NEW:
                showInvite(AJS.I18n.getText("me.getscreen.web.button.waiting"), AJS.I18n.getText("me.getscreen.web.status.waiting"));
                showBalloon(STATUS_NEW, AJS.I18n.getText("me.getscreen.web.baloon.waiting"));
                break;

            case STATUS_CONNECTED:
                showInvite(AJS.I18n.getText("me.getscreen.web.button.connect"), AJS.I18n.getText("me.getscreen.web.status.connected"));
                showBalloon(STATUS_CONNECTED, AJS.I18n.getText("me.getscreen.web.baloon.connected"));
                break;

            case STATUS_ACTIVE:
                showInvite(AJS.I18n.getText("me.getscreen.web.button.connect"), AJS.I18n.getText("me.getscreen.web.status.active"));
                showBalloon(STATUS_ACTIVE, AJS.I18n.getText("me.getscreen.web.baloon.active"));
                break;
        }
        supportStatus = status;
    }

    function onStartClick(event) {
        switch (supportStatus) {
            case STATUS_UNKNOWN:
                requestGet();
                break;
            case STATUS_NONE:
            case STATUS_CLOSED:
                requestAction("create");
                break;
            case STATUS_NEW:
            case STATUS_CONNECTED:
                if (connectUrl && connectUrl != "")
                    window.open(connectUrl, "_blank");
                break;
        }
        event.preventDefault();
    }

    function onCancelClick(event) {
        switch (supportStatus) {
            case STATUS_NEW:
            case STATUS_CONNECTED:
            case STATUS_ACTIVE:
                requestAction("delete");
                break;
        }
        event.preventDefault();
    }
});
